"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveAllAssetFiles = exports.resolveAllPaperclipFiles = void 0;
var path = require("path");
var glob = require("glob");
var paperclip_utils_1 = require("paperclip-utils");
// TODO - move to paperclip-utils as soon as we have a glob library that can handle virtual file systems
var findResourcesFromConfig = function (get) { return function (fs) { return function (fromUri, relative) {
    // symlinks may fudge module resolution, so we need to find the real path
    var fromPath = fs.realpathSync(new URL(fromUri).pathname);
    var fromPathDirname = path.dirname(fromPath);
    var configUrl = paperclip_utils_1.findPCConfigUrl(fs)(fromUri);
    // If there's no config, then don't bother looking for PC files. Otherwise we're likely
    // to need esoteric logic for resolving PC that I don't think should be supported -- there should
    // just be aproach.
    if (!configUrl) {
        return [];
    }
    var configPath = new URL(configUrl).pathname;
    var config = JSON.parse(fs.readFileSync(configPath, "utf8"));
    return get(config, path.dirname(configPath))
        .filter(function (pathname) { return pathname !== fromPath; })
        .map(function (pathname) {
        if (relative) {
            var modulePath = getModulePath(configUrl, config, pathname, fromPathDirname);
            if (modulePath.charAt(0) !== "/") {
                return modulePath;
            }
            var relativePath = path.relative(fromPathDirname, modulePath);
            if (relativePath.charAt(0) !== ".") {
                relativePath = "./" + relativePath;
            }
            return relativePath;
        }
        return "file://" + pathname;
    });
}; }; };
exports.resolveAllPaperclipFiles = findResourcesFromConfig(function (config, cwd) {
    return glob.sync(paperclip_utils_1.paperclipSourceGlobPattern(config.sourceDirectory), {
        cwd: cwd,
        realpath: true
    });
});
exports.resolveAllAssetFiles = findResourcesFromConfig(function (config, cwd) {
    var ext = "+(jpg|jpeg|png|gif|svg)";
    var sourceDir = config.sourceDirectory;
    if (sourceDir === ".") {
        return glob.sync("**/*." + ext, { cwd: cwd, realpath: true });
    }
    return glob.sync(sourceDir + "/**/*." + ext, { cwd: cwd, realpath: true });
});
var getModulePath = function (configUri, config, fullPath, fromDir) {
    var configDir = path.dirname(new URL(configUri).pathname);
    var moduleDirectory = path.join(configDir, config.sourceDirectory) + "/";
    if (fullPath.indexOf(moduleDirectory) === 0) {
        var modulePath = fullPath.replace(moduleDirectory, "");
        var nextDirectory = path.join(moduleDirectory, modulePath.split("/")[0]) + "/";
        if (!fromDir || fromDir.indexOf(nextDirectory) === -1) {
            return modulePath;
        }
    }
    return fullPath;
};
